package com.kpro.algorithm;

/**
 * A distance metric that calculates distance based on 
 * weighed union of a bit map interception
 *
 * @author Dimitry Kongevold(dimitryk)
 * 
 */


import java.util.ArrayList;
import java.util.Properties;

import com.kpro.dataobjects.Case;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.dataobjects.Purpose;
import com.kpro.dataobjects.Recipient;
import com.kpro.dataobjects.Retention;

/**
 * A distance metric that calculates distance based on 
 * weighed union of a bit map interception
 * @version 240911.01 
 * @author dimitryk
 */
public class Bitmapwithdata  extends DistanceMetric{
	private double ours, delivery, same, other_recipient, unrelated, pub;
	private double current, admin, develop, tailoring, pseudo_analysis, pseudo_decision;
	private double individual_analysis, individual_decision, contact, historical, telemarketing, other_purpose;
	private double no_retention, stated_purpose, legal_requirement, business_practices, indefinitely;

	Bitmapwithdata(Properties weights, String[] extraArgs){
		super(weights, extraArgs);
	}

	private void setWeights(){
		ours=Double.parseDouble(weightsConfig.getProperty("Recipient.OURS"));
		delivery=Double.parseDouble(weightsConfig.getProperty("Recipient.DELIVERY"));
		same=Double.parseDouble(weightsConfig.getProperty("Recipient.SAME"));
		other_recipient=Double.parseDouble(weightsConfig.getProperty("Recipient.OTHER_RECIPIENT"));
		unrelated=Double.parseDouble(weightsConfig.getProperty("Recipient.UNRELATED"));
		pub=Double.parseDouble(weightsConfig.getProperty("Recipient.PUBLIC"));

		current=Double.parseDouble(weightsConfig.getProperty("Purpose.CURRENT"));
		admin=Double.parseDouble(weightsConfig.getProperty("Purpose.ADMIN"));
		develop= Double.parseDouble(weightsConfig.getProperty("Purpose.DEVELOP"));
		tailoring=Double.parseDouble(weightsConfig.getProperty("Purpose.TAILORING"));
		pseudo_analysis=Double.parseDouble(weightsConfig.getProperty("Purpose.PSEUDO_ANALYSIS"));
		pseudo_decision=Double.parseDouble(weightsConfig.getProperty("Purpose.PSEUDO_DECISION"));
		individual_analysis=Double.parseDouble(weightsConfig.getProperty("Purpose.INDIVIDUAL_ANALYSIS"));
		individual_decision=Double.parseDouble(weightsConfig.getProperty("Purpose.INDIVIDUAL_DECISION"));
		contact=Double.parseDouble(weightsConfig.getProperty("Purpose.CONTACT"));
		historical=Double.parseDouble(weightsConfig.getProperty("Purpose.HISTORICAL"));
		telemarketing=Double.parseDouble(weightsConfig.getProperty("Purpose.TELEMARKETING"));
		other_purpose=Double.parseDouble(weightsConfig.getProperty("Purpose.OTHER_PURPOSE"));

		no_retention=Double.parseDouble(weightsConfig.getProperty("Retention.NO_RETENTION"));
		stated_purpose=Double.parseDouble(weightsConfig.getProperty("Retention.STATED_PURPOSE"));
		legal_requirement=Double.parseDouble(weightsConfig.getProperty("Retention.LEGAL_REQUIREMENT"));
		business_practices=Double.parseDouble(weightsConfig.getProperty("Retention.BUSINESS_PRACTICES"));
		indefinitely=Double.parseDouble(weightsConfig.getProperty("Retention.INDEFINITELY"));

	}

	/**
	 * Internal method that makes a bit map of 6 ints
	 * each place in an array corresponds to a specific value
	 * of a Recipient field
	 *  
	 * @author dimitryk
	 * @param list input ArrayList of Recipients for method.
	 * @return Map as bitmap
	 */	
	private double[] makeRecipientMap(ArrayList<Recipient> list) {
		double[] Map = new double[6];
		
		for(int d=0; d < 6; d++)
			Map[d]=0;

		for(int i=0; i < list.size(); i++) {
			switch (list.get(i)) {
			case OURS: //weights can come in here 2
				Map[0]= ours;
				break;
			case DELIVERY:			
				Map[1]=delivery;
				break;
			case SAME:
				Map[2]=same;
				break;
			case OTHER_RECIPIENT:
				Map[3]=other_recipient;
				break;
			case UNRELATED:
				Map[4]=unrelated;
				break;
			case PUBLIC:
				Map[5]=pub;
				break;
			default:break;
			}
		}
		return Map;
	}

	/**
	 * internal method that makes a bit map of 12 doubles
	 * each place in an array corresponds to a spesfic value
	 * of a Purpose Enum
	 * @author dimitryk
	 * @param list input arraylist of Purpose for method...
	 * @return Map as bitmap
	 */
	private double[] makePurposeMap(ArrayList<Purpose> list){
		double[] Map = new double[12];
		
		for(int d=0; d < 12; d++)
			Map[d]=0;
		
		for(int i=0; i < list.size(); i++) {
			switch (list.get(i)) {
			case CURRENT:
				Map[0]=current;
				break;
			case ADMIN:
				Map[1]=admin;
				break;
			case DEVELOP:
				Map[2]=develop;
				break;
			case TAILORING:
				Map[3]=tailoring;
				break;
			case PSEUDO_ANALYSIS:
				Map[4]=pseudo_analysis;
				break;
			case PSEUDO_DECISION:
				Map[5]=pseudo_decision;
				break;
			case INDIVIDUAL_ANALYSIS:
				Map[6]=individual_analysis;
				break;
			case INDIVIDUAL_DECISION:
				Map[7]=individual_decision;
				break;
			case CONTACT:
				Map[8]=contact;
				break;
			case HISTORICAL:
				Map[9]=historical;
				break;
			case TELEMARKETING:
				Map[10]=telemarketing;
				break;
			case OTHER_PURPOSE:
				Map[11]=other_purpose;
				break;
			default:break;
			}
		}
		return Map;
	}

	/**
	 * internal method that makes a bit map of 5 ints
	 * each place in an array corresponds to a spesfic value
	 * of a Retention Enum
	 * @author dimitryk
	 * @param list input arraylist of Purpose for method...
	 * @return Map as bitmap
	 */

	private double[] MakeRetentionMap(ArrayList<Retention> list){
		double[] Map = new double[5];
		
		for(int d=0; d < 5; d++)
			Map[d]=0;
		
		for(int i=0; i < list.size(); i++) {
			switch (list.get(i)){
			case NO_RETENTION: //weights can come in here 2
				Map[0]=no_retention;
				break;
			case STATED_PURPOSE:
				Map[1]=stated_purpose;
				break;
			case LEGAL_REQUIREMENT:
				Map[2]=legal_requirement;
				break;
			case BUSINESS_PRACTICES:
				Map[3]=business_practices;
				break;
			case INDEFINITELY:
				Map[4]=indefinitely;
				break;
			default:break;
			}
		}
		return Map;
	}

	/**
	 * Calculates distance in the Recipient field from case a to case b
	 * using bit map distance = a interception b * weight
	 * 
	 * @author dimitryk
	 * @param a input case.
	 * @param b input case.
	 * @return distance double for the distance between to.
	 */

	private double getRecipientDistance(Case a, Case b) {
		double[] MapA, MapB;
		MapA = makeRecipientMap(a.getRecipients());
		MapB = makeRecipientMap(b.getRecipients());
		double distance = 0;

		for(int i=0; i < 6; i++) {
			if (MapA[i] != MapB[i]) {
				distance = distance + Math.max(MapA[i], MapB[i]);
				//dis=dis+1; //for now... weights are jet not operational.
			}
		}
		return distance * Double.parseDouble(weightsConfig.getProperty("recipient")); // dis*weight later
	}

	/**
	 * Calculates distance in the Retention field from case a to case b
	 * using bit map distance = a interseption b * weight
	 * @author dimitryk
	 * @param a input case.
	 * @param b input case.
	 * @return dis double for the distance between to.
	 */

	private double getRetentionDistance(Case a, Case b) {
		double[] MapA, MapB;
		MapA = MakeRetentionMap(a.getRetentions());
		MapB = MakeRetentionMap(b.getRetentions());
		double distance = 0;

		for (int i=0; i < 5; i++) {
			if (MapA[i] != MapB[i]) {
				distance = distance + Math.max(MapA[i], MapB[i]);	
			}
		}
		return distance * Double.parseDouble(weightsConfig.getProperty("retention"));// dis*weight later
	}

	private double getDataDistance(Case a, Case b) {
		String[] DataStringsA = a.getDataType().split(".");
		String[] DataStringsB = b.getDataType().split(".");
		int LastSameString=0;
		
		/*
		 * next for-loop only goes for the length of least of strings
		 * obviosly they can be same after shortest array integrated
		 * loot increases LastSameWord with 1 if the string in A same as in B
		 * and breaks at first mismach of A and B
		 */
		for(int i=0; i < Math.min(DataStringsA.length, DataStringsB.length); i++) { 
			if (!(DataStringsA[i].equals(DataStringsB[i])))
				break;		
			LastSameString++;
		}
		
		/*
		 * returns lenght of "tails" that did not matched
		 */
		return DataStringsA.length + DataStringsA.length + 2*LastSameString;
	}
	
	/**
	 * Calculates distance in the Purpose field from case a to case b
	 * using bit map distance = a interseption b * weight
	 * 
	 * @author dimitryk
	 * @param a input case.
	 * @param b input case.
	 * @return dis double for the distance between to.
	 */

	private double getPurposeDistance(Case a, Case b) {
		double[] MapA, MapB;
		MapA = makePurposeMap(a.getPurposes());
		MapB = makePurposeMap(b.getPurposes());
		double distance = 0;

		for(int i=0; i < 12; i++) {
			if (MapA[i] != MapB[i]) {
				distance = distance + Math.max(MapA[i], MapB[i]);
			}
		}
		return distance*Double.parseDouble(weightsConfig.getProperty("purpose"));// dis*weight later
	}

	/**
	 * Returns the distance between two cases a and b based on purpose, recipient, retention, and data.
	 * @author dimitryk
	 * @param a the 1st case 
	 * @param b the 2nd case
	 * @return the distance between cases 
	 */
	private double getSumDistance(Case a, Case b) {
		return getPurposeDistance(a,b) + 
				getRecipientDistance(a,b) + 
				getRetentionDistance(a,b) + 
				getDataDistance(a,b);
	}

	/**
	 * Returns the distance between two PolicyObjects, summing the minimum distance from B to A and 
	 * then the distance from A to B.
	 * 
	 * @param a the 1st policy object
	 * @param b the second policy object
	 * @param time the direction to run- if not called from inside method, it should be 0
	 * @return the distance between the two policy objects
	 */
	private double getDistance(PolicyObject a, PolicyObject b, int time){
		long start = System.currentTimeMillis();
		ArrayList<Case> casesA, casesB;
		casesA = a.getCases();
		casesB = b.getCases();
		double minimumDistance;
		double distance = 0;
		
		if (time == 0) {
			distance += getDistance(b,a,1);  //first B Vs A then A vs B
		}

		for (int i = 0; i < casesA.size(); i++) {
			minimumDistance = Double.MAX_VALUE;
			
			for (int d = 0; d < casesB.size(); d++) {
				minimumDistance = Math.min(getSumDistance(casesA.get(i), 
						casesB.get(d)),minimumDistance);
			}
			distance += minimumDistance;
		}
		long end = System.currentTimeMillis();
		System.out.println("Execution speed "+ (end - start)+" ms\n");
		return distance;
	}

	/**
	 * Initializes weights and returns the distance between two PolicyObjects.
	 * @param a the 1st policy object
	 * @param b the second policy object
	 * @return the distance between the two policy objects
	 */
	@Override
	public double getTotalDistance(PolicyObject a, PolicyObject b) {
		setWeights();
		return getDistance(a,b,0);
	}
}