package com.kpro.algorithm;
import com.kpro.dataobjects.Action;
import com.kpro.dataobjects.PolicyObject;

/**
 * abstract class for all conclusion classes (they take the new policy and 
 * a reduction of the history versus new policy), and return an Action.
 * May extend ReductionAlgorithm in the future (return a modified np, instead of
 * an action).
 * May be used instead of a ReductionAlgorithm. call with:
 * Action a = (new ConclusionAlgorithm()).conclude(newpol,theIO.getPDB());
 * 
 * @author ngerstle
 * @version 29.09.11.1
 * 
 */

public abstract class ConclusionAlgorithm {
	
	protected DistanceMetric distanceMetric;
	String[] extraArgs = null;
	
	/**
	 * ConclustionAlgorithm constructor
	 * @param distanceMetric 
	 * @param extraArgs
	 */
	public ConclusionAlgorithm(DistanceMetric distanceMetric, String[] extraArgs){
		this.distanceMetric = distanceMetric;
		this.extraArgs = extraArgs;
	}

	/**
	 * Provides an action recommendation for the new policy based on the given set of objects
	 * @param policyObject the new policy
	 * @param kNearest a set of relevant policies
	 * @return a recommended Action
	 */
	public abstract Action conclude(PolicyObject policyObject, Iterable<PolicyObject> kNearest);



}
