package com.kpro.algorithm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import com.kpro.dataobjects.Case;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.datastorage.PolicyDatabase;

public class KNNRetrieval extends ReductionAlgorithm {
	
	private DistanceMetric distanceMetric;
	private int k = 1;

	public KNNRetrieval(DistanceMetric distanceMetric, PolicyDatabase policyDatabase, String[] extraArgs) {
		super(policyDatabase, extraArgs);
		this.distanceMetric = distanceMetric;
		this.k = Integer.parseInt(extraArgs[0]);
	}
	
	private Map.Entry<PolicyObject, Double> getLargest(Map<PolicyObject, Double> nearest) {
		double largest = -1;
		Map.Entry<PolicyObject, Double> largestPolicyObject = null;
		for (Map.Entry<PolicyObject, Double> entry : nearest.entrySet()) {
			if (entry.getValue() > largest) {
				largest = entry.getValue();
				largestPolicyObject = entry;
			}
		}
		return largestPolicyObject;
	}

	@Override
	public ArrayList<PolicyObject> reduce(PolicyObject currentPolicy) {
		
		Map<PolicyObject, Double> nearest = new HashMap<PolicyObject, Double>();
		PolicyObject largestNearestPolicy = null;
		double largestNearestDistance = 0;
		
		for (PolicyObject previousPolicy : policyDatabase) {
			long start = System.currentTimeMillis();
			double distance = distanceMetric.getTotalDistance(currentPolicy, previousPolicy);
			long end = System.currentTimeMillis();
			System.out.println((end-start));
//			System.out.println(previousPolicy.getCases().size());
			
			if (nearest.size() < k) {
				nearest.put(previousPolicy, distance);
				
				Map.Entry<PolicyObject, Double> largestNearest = getLargest(nearest);
				largestNearestPolicy = largestNearest.getKey();
				largestNearestDistance = largestNearest.getValue();
			} else {
				if (distance < largestNearestDistance) {
					nearest.remove(largestNearestPolicy);
					nearest.put(previousPolicy, distance);
				
					Map.Entry<PolicyObject, Double> largestNearest = getLargest(nearest);
					largestNearestPolicy = largestNearest.getKey();
					largestNearestDistance = largestNearest.getValue();
				}
			}
		}
		
		ArrayList<PolicyObject> result = new ArrayList<PolicyObject>();
		for (PolicyObject policy : nearest.keySet()) {
			result.add(policy);
		}
		
		return result;
	}
}