package com.kpro.algorithm;

import java.util.Map.Entry;
import java.util.Properties;

import net.sourceforge.jFuzzyLogic.fcl.FclParser.main_return;

import com.kpro.dataobjects.Case;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.dataobjects.Purpose;
import com.kpro.dataobjects.Recipient;
import com.kpro.dataobjects.Retention;
import com.kpro.datastorage.PolicyDatabase;
import com.kpro.main.Gio;

public class LearnAlgImportance extends LearnAlgorithm {

	private static final String PURPOSE = "Purpose";
	private static final String RETENTION = "Retention";
	private static final String RECIPIENT = "Recipient";

	private PolicyDatabase policyDatabase;

	public LearnAlgImportance(Properties weightsConfig, String[] extraArgs) {
		super(weightsConfig, extraArgs);
	}

	private static double calculateWeight(double rejectedPolicies, double numberOfPolicies) {
		if (numberOfPolicies == 0)
			return 0;
		double z = 1.95;
		double phat = 1.0*rejectedPolicies/numberOfPolicies;
		return (phat + z*z/(2*numberOfPolicies) 
				- z * Math.sqrt((phat*(1-phat)+z*z/(4*numberOfPolicies))
						/numberOfPolicies))/(1+z*z/numberOfPolicies);
	}

	private double getElementImportance(Object policyElement) {
		double rejectionCount = 0;
		double acceptanceCount = 0;
		boolean containsElement = false;

		for (PolicyObject policyObject : policyDatabase) {
			for (Case statement : policyObject.getCases()) {
				if (!containsElement && policyElement.toString().matches("^" + PURPOSE + ".*")) {
					for (Purpose purpose : statement.getPurposes()) {
						if (policyElement.toString().substring(8).equals(purpose.toString())) {
							containsElement = true;
							break;
						}
					}
				}
				else if (!containsElement && policyElement.toString().matches("^" + RETENTION + ".*")) {
					for (Retention retention : statement.getRetentions()) {
						if (policyElement.toString().substring(10).equals(retention.toString())) {
							containsElement = true;
							break;
						}
					}
				}
				else if (!containsElement && policyElement.toString().matches("^" + RECIPIENT + ".*")) {
					for (Recipient recipient : statement.getRecipients()) {
						if (policyElement.toString().substring(10).equals(recipient.toString())) {
							containsElement = true;
							break;
						}
					}
				}
				else if (!containsElement && statement.getDataType().equals("#"+policyElement.toString())) {
					containsElement = true;
				}
			}
			if (containsElement) {
				if (policyObject.getAction() != null) {
					if (!policyObject.getAction().getAccepted()) {
						rejectionCount++;
					} else {
						acceptanceCount++;
					}				
				}
			}
		}
		return calculateWeight(rejectionCount, rejectionCount+acceptanceCount);
	}

	@Override
	protected Properties applyML(Gio theIO) {
		Properties weights = theIO.getWeights();
		policyDatabase = theIO.getPolicyDatabase();
		Properties newWeights = new Properties();

		for(Object policyElement : weights.keySet()) {
			newWeights.setProperty(policyElement.toString(), 
					Double.toString(getElementImportance(policyElement)));
		}

		for (Entry<Object, Object> entry : newWeights.entrySet()) {
			System.out.println(entry.getKey() + " = " + entry.getValue());
		}

		return newWeights;
	}
}