package com.kpro.algorithm;

import java.util.Properties;

import com.kpro.dataobjects.Case;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.dataobjects.Purpose;
import com.kpro.dataobjects.Recipient;
import com.kpro.dataobjects.Retention;
import com.kpro.datastorage.PolicyDatabase;
import com.kpro.main.Gio;

/**
 * An learning algorithm that extends LearnAlgorithm
 * 
 * @author Nesha
 *
 */
public class LearnAlgStand extends LearnAlgorithm {

	private PolicyDatabase policyDatabase;

	/**
	 * constructor
	 * @param weightsConfig the weights
	 * @param extraArgs from the config
	 */
	public LearnAlgStand(Properties weightsConfig, String[] extraArgs) {
		super(weightsConfig, extraArgs);
	}

	/**
	 * Goes through the whole weights Properties and sets new
	 * weights which it gets by calling correlation with every
	 * key in the property as its input. 
	 * 
	 * @param theIO the GIO
	 * @return Properties the new weights
	 */
	protected Properties applyML(Gio theIO) {
		Properties weights = theIO.getWeights();
		policyDatabase = theIO.getPolicyDatabase();
		Properties newWeights = new Properties();

		for(Object i : weights.keySet()) {
			newWeights.setProperty(i.toString(), Double.toString(value(correlation(i))));
		}
		return newWeights;
	}

	/**
	 * Called by applyML()
	 * 
	 * @param x double value between [0,1] which is countyes/countno from applyML()
	 * @return double value [0,5] that represents a new weights value
	 */
	private double value(double x){
		return 10*Math.abs(x-0.5);
	}

	/**
	 * Goes through the PolicyObjects and their cases and checks if 
	 * at least one of them contains the parameter. And if it does, the
	 * returned value is set to the % of the amount of the PolicyObjects 
	 * (that contains this parameter) that have their actions accepted.
	 * If the parameter isn't in any of the PolicyObjects the returned 
	 * value is 0.5  
	 * 
	 * @param Objects From GIO.getWeights()
	 * @return double
	 */
	private double correlation(Object i){
		double countYes = 1;
		double countNo = 1;
		boolean y = false;
		
		for(PolicyObject po : policyDatabase) {
			//checks if the parameter object is in any of the cases in any of the policyobjects
			for(Case statement : po.getCases()) {
				if(!y && (i.toString().length() > 8)) {
					for(Purpose purpose : statement.getPurposes()){
						//String compares the purposes from the config file, and the ones stored in the database
						if(i.toString().substring(8).equals(purpose.toString())){
							//if y is set to true we don't need to check further
							y = true;
							break;
						}
					}
				}
				if(!y && (i.toString().length() > 10)) {
					for(Retention retention : statement.getRetentions()) {
						if(i.toString().substring(10).equals(retention.toString())) {
							y = true;
							break;
						}
					}
				}
				if(!y && (i.toString().length() > 10)) {
					for(Recipient recipient : statement.getRecipients()){
						if(i.toString().substring(10).equals(recipient.toString())) {
							y = true;
							break;
						}
					}
				}
			}

			if(y){
				if(po.getAction() == null){
					//System.out.println("PolicyObject.getAction == null in LearnAlgSimpler");
				} else {
					if(po.getAction().getAccepted()) {
						countYes ++;
					} else {
						countNo ++;
					}
				}
			}
		}
		return countYes/(countYes+countNo);
	}
}