package com.kpro.algorithm;

import java.util.ArrayList;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.datastorage.PolicyDatabase;

/**
 * The abstract class for implementing reduction algorithms, like kNN.
 * ReductionAlgorithm objects store the database, and reduce the set of polices to
 * only the relevant policies (one or more).
 * May include 'Conclusion'/'Summary' algorithms in the future.
 * 
 * @author ngerstle
 * @version 29.09.11.1
 *
 */
public abstract class ReductionAlgorithm {

	protected PolicyDatabase policyDatabase; //the history we are reducing
	String[] extraArgs = null;
	
	/**
	 * Constructor for a reductionAlgorithm
	 * 
	 * @param policyDatabase
	 * @param extraArgs
	 */
	public ReductionAlgorithm(PolicyDatabase policyDatabase, String[] extraArgs)
	{
		this.policyDatabase = policyDatabase;
		this.extraArgs = extraArgs;
	}

	/**
	 * The reduce call. returns an ArrayList of policies in the PolicyDatabase relevant to newPO
	 * 
	 * @param policyDatabase 
	 * @param policyObject The new policy to consider- it shouldn't change within the algorithm
	 * @return a modified policyObject 
	 * @author ngerstle
	 */
	public abstract ArrayList<PolicyObject> reduce(final PolicyObject policyObject);

}