package com.kpro.algorithm;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.sourceforge.jFuzzyLogic.FIS;

import com.kpro.dataobjects.Case;
import com.kpro.dataobjects.PolicyObject;
import com.kpro.dataobjects.Purpose;
import com.kpro.dataobjects.Recipient;
import com.kpro.dataobjects.Retention;

public class FuzzyDistance extends DistanceMetric {

	private static final String FCL_FILENAME = "policy.fcl";

	private FIS fis = null;

	private double ours, delivery, same, other_recipient, unrelated, pub;
	private double current, admin, develop, tailoring, pseudo_analysis, pseudo_decision;
	private double individual_analysis, individual_decision, contact, historical, telemarketing, other_purpose;
	private double no_retention, stated_purpose, legal_requirement, business_practices, indefinitely;

	protected FuzzyDistance(Properties weights, String[] extraArgs) {
		super(weights, extraArgs);

		String fileName = FCL_FILENAME;
		fis = FIS.load(fileName,true);

		if( fis == null ) { 
			System.err.println("Can't load file: '" 
					+ fileName + "'");
			return;
		}
	}

	private void setWeights(){
		ours=Double.parseDouble(weightsConfig.getProperty("Recipient.OURS"));
		delivery=Double.parseDouble(weightsConfig.getProperty("Recipient.DELIVERY"));
		same=Double.parseDouble(weightsConfig.getProperty("Recipient.SAME"));
		other_recipient=Double.parseDouble(weightsConfig.getProperty("Recipient.OTHER_RECIPIENT"));
		unrelated=Double.parseDouble(weightsConfig.getProperty("Recipient.UNRELATED"));
		pub=Double.parseDouble(weightsConfig.getProperty("Recipient.PUBLIC"));

		current=Double.parseDouble(weightsConfig.getProperty("Purpose.CURRENT"));
		admin=Double.parseDouble(weightsConfig.getProperty("Purpose.ADMIN"));
		develop= Double.parseDouble(weightsConfig.getProperty("Purpose.DEVELOP"));
		tailoring=Double.parseDouble(weightsConfig.getProperty("Purpose.TAILORING"));
		pseudo_analysis=Double.parseDouble(weightsConfig.getProperty("Purpose.PSEUDO_ANALYSIS"));
		pseudo_decision=Double.parseDouble(weightsConfig.getProperty("Purpose.PSEUDO_DECISION"));
		individual_analysis=Double.parseDouble(weightsConfig.getProperty("Purpose.INDIVIDUAL_ANALYSIS"));
		individual_decision=Double.parseDouble(weightsConfig.getProperty("Purpose.INDIVIDUAL_DECISION"));
		contact=Double.parseDouble(weightsConfig.getProperty("Purpose.CONTACT"));
		historical=Double.parseDouble(weightsConfig.getProperty("Purpose.HISTORICAL"));
		telemarketing=Double.parseDouble(weightsConfig.getProperty("Purpose.TELEMARKETING"));
		other_purpose=Double.parseDouble(weightsConfig.getProperty("Purpose.OTHER_PURPOSE"));

		no_retention=Double.parseDouble(weightsConfig.getProperty("Retention.NO_RETENTION"));
		stated_purpose=Double.parseDouble(weightsConfig.getProperty("Retention.STATED_PURPOSE"));
		legal_requirement=Double.parseDouble(weightsConfig.getProperty("Retention.LEGAL_REQUIREMENT"));
		business_practices=Double.parseDouble(weightsConfig.getProperty("Retention.BUSINESS_PRACTICES"));
		indefinitely=Double.parseDouble(weightsConfig.getProperty("Retention.INDEFINITELY"));

	}

	private Map<Retention, Double> getRetentionWeights(ArrayList<Retention> retentions) {
		Map<Retention, Double> retentionWeights = new HashMap<Retention, Double>();

		for(int i=0; i < retentions.size(); i++) {
			switch (retentions.get(i)){
			case NO_RETENTION:
				retentionWeights.put(Retention.NO_RETENTION, no_retention);
				break;
			case STATED_PURPOSE:
				retentionWeights.put(Retention.STATED_PURPOSE, stated_purpose);
				break;
			case LEGAL_REQUIREMENT:
				retentionWeights.put(Retention.LEGAL_REQUIREMENT, legal_requirement);
				break;
			case BUSINESS_PRACTICES:
				retentionWeights.put(Retention.BUSINESS_PRACTICES, business_practices);
				break;
			case INDEFINITELY:
				retentionWeights.put(Retention.INDEFINITELY, indefinitely);
				break;
			default:break;
			}
		}
		return retentionWeights;
	}

	private double getRetentionDistance(Case current, Case previous) {
		Map<Retention, Double> currentRetentions = getRetentionWeights(current.getRetentions());
		Map<Retention, Double> oldRetentions = getRetentionWeights(previous.getRetentions());

		double distance = 0;
		for (Map.Entry<Retention, Double> currentEntry : currentRetentions.entrySet()) {
			for (Map.Entry<Retention, Double> previousEntry : oldRetentions.entrySet()) {
				if (currentEntry.getKey() != previousEntry.getKey()) {
					fis.setVariable("current_element", currentEntry.getValue());
					fis.setVariable("previous_element", previousEntry.getValue());
					fis.evaluate();
					distance += fis.getVariable("similarity").defuzzify();
				}				
			}
		}
		return distance;
	}

	private Map<Recipient, Double> getRecipientWeights(ArrayList<Recipient> recipients) {
		Map<Recipient, Double> recipientWeights = new HashMap<Recipient, Double>();

		for(int i=0; i < recipients.size(); i++) {
			switch (recipients.get(i)) {
			case OURS:
				recipientWeights.put(Recipient.OURS, ours);
				break;
			case DELIVERY:
				recipientWeights.put(Recipient.DELIVERY, delivery);
				break;
			case SAME:
				recipientWeights.put(Recipient.SAME, same);
				break;
			case OTHER_RECIPIENT:
				recipientWeights.put(Recipient.OTHER_RECIPIENT, other_recipient);
				break;
			case UNRELATED:
				recipientWeights.put(Recipient.UNRELATED, unrelated);
				break;
			case PUBLIC:
				recipientWeights.put(Recipient.PUBLIC, pub);
				break;
			default:break;
			}
		}
		return recipientWeights;
	}

	private double getRecipientDistance(Case current, Case previous) {
		Map<Recipient, Double> currentRecipient = getRecipientWeights(current.getRecipients());
		Map<Recipient, Double> oldRecipient = getRecipientWeights(previous.getRecipients());

		double distance = 0;
		for (Map.Entry<Recipient, Double> currentEntry : currentRecipient.entrySet()) {
			for (Map.Entry<Recipient, Double> previousEntry : oldRecipient.entrySet()) {
				if (currentEntry.getKey() != previousEntry.getKey()) {
					fis.setVariable("current_element", currentEntry.getValue());
					fis.setVariable("previous_element", previousEntry.getValue());
					fis.evaluate();
					distance += fis.getVariable("similarity").defuzzify();
				}				
			}
		}
		return distance;
	}

	private Map<Purpose, Double> getPurposeWeights(ArrayList<Purpose> purposes) {
		Map<Purpose, Double> purposeWeights = new HashMap<Purpose, Double>();

		for(int i = 0; i < purposes.size(); i++) {
			switch (purposes.get(i)) {
			case CURRENT:
				purposeWeights.put(Purpose.CURRENT, current);
				break;
			case ADMIN:
				purposeWeights.put(Purpose.ADMIN, admin);
				break;
			case DEVELOP:
				purposeWeights.put(Purpose.DEVELOP, develop);
				break;
			case TAILORING:
				purposeWeights.put(Purpose.TAILORING, tailoring);
				break;
			case PSEUDO_ANALYSIS:
				purposeWeights.put(Purpose.PSEUDO_ANALYSIS, pseudo_analysis);
				break;
			case PSEUDO_DECISION:
				purposeWeights.put(Purpose.PSEUDO_DECISION, pseudo_decision);
				break;
			case INDIVIDUAL_ANALYSIS:
				purposeWeights.put(Purpose.INDIVIDUAL_ANALYSIS, individual_analysis);
				break;
			case INDIVIDUAL_DECISION:
				purposeWeights.put(Purpose.INDIVIDUAL_DECISION, individual_decision);
				break;
			case CONTACT:
				purposeWeights.put(Purpose.CONTACT, contact);
				break;
			case HISTORICAL:
				purposeWeights.put(Purpose.HISTORICAL, historical);
				break;
			case TELEMARKETING:
				purposeWeights.put(Purpose.TELEMARKETING, telemarketing);
				break;
			case OTHER_PURPOSE:
				purposeWeights.put(Purpose.OTHER_PURPOSE, other_purpose);
				break;
			default:
				break;
			}
		}
		return purposeWeights;
	}

	private int findDatatypeLevel(String datatype) {
		int maxLevel = 5;
		System.out.println("String: " + datatype);
		String[] dataTypes = datatype.split("\\.");
		
		return maxLevel-(dataTypes.length-1);
	}

	private double getDataDistance(String current, String previous) {
		if (current.equals(previous)) 
			return 0;
		
		if (previous.matches("#userdetail.*") 
				|| previous.matches(".*referrer") 
				|| previous.matches("#others.job-application-and-email-preference")) 
			return 0;
		
		double distance = 0;
		//System.out.println(current + " <-> " + previous);
		if (current.matches("#.*") && previous.matches("#.*")) {
			double currentWeight = Double.parseDouble(weightsConfig.getProperty(current.substring(1)));
			double previousWeight = Double.parseDouble(weightsConfig.getProperty(previous.substring(1)));			
			fis.setVariable("current_element", currentWeight);
			fis.setVariable("previous_element", previousWeight);
			fis.evaluate();			
			distance = fis.getVariable("similarity").defuzzify();
		}
		//System.out.println("Distance: " + distance);
		return distance;
	}

	private double getPurposeDistance(Case current, Case previous) {
		Map<Purpose, Double> currentPurposes = 
				getPurposeWeights(current.getPurposes());
		Map<Purpose, Double> previousPurposes = 
				getPurposeWeights(previous.getPurposes());

		Iterator currentIterator = currentPurposes.entrySet().iterator();
		Set<Map.Entry<Purpose, Double>> previousPurposeSet = previousPurposes.entrySet();

		double distance = 0;
		while (currentIterator.hasNext()) {
			Map.Entry<Purpose, Double> currentEntry = 
					(Map.Entry<Purpose, Double>)currentIterator.next();

			for (Map.Entry<Purpose, Double> entry : previousPurposeSet) {
				if (currentEntry.getKey() != entry.getKey()) {
					fis.setVariable("current_element", currentEntry.getValue());
					fis.setVariable("previous_element", entry.getValue());
					fis.evaluate();
					distance += fis.getVariable("similarity").defuzzify();
				}
			}
		}
		return distance;
	}

	@Override
	public double getTotalDistance(PolicyObject currentPolicy, 
			PolicyObject previousPolicy) {
		long start = System.currentTimeMillis();
		setWeights();

		double purposeDistance = 0;
		double recipientDistance = 0;
		double retentionDistance = 0;
		double dataDistance = 0;

		ArrayList<Case> currentStatements = currentPolicy.getCases(); 
		ArrayList<Case> previousStatements = previousPolicy.getCases();

		for (Case currentStatement : currentStatements) {
			for (Case previousStatement : previousStatements) {
				purposeDistance +=  
						getPurposeDistance(currentStatement, previousStatement);
				recipientDistance += 
						getRecipientDistance(currentStatement, previousStatement);
				retentionDistance += 
						getRetentionDistance(currentStatement, previousStatement);
				dataDistance +=
						getDataDistance(currentStatement.getDataType(), 
								previousStatement.getDataType());
			}
		}
		long end = System.currentTimeMillis();
//		System.out.println("Distance: " + (purposeDistance + recipientDistance + retentionDistance + dataDistance));
//		System.out.println(end-start + " ms");
//		System.out.println("*****");
		return purposeDistance + recipientDistance + retentionDistance + dataDistance;
	}
}